/*
 * ADC.c
 *
 * Created: 08.02.2015 15:01:34
 *  Author:  Stanislav Chystiakov (DESY)
 */ 
#include <stdint.h>
#include "port_init.h"
#include "timer.h"
#include "ADC.h"
#include <util/delay.h>
#include <avr/io.h>
#include <avr/interrupt.h>

uint8_t samplesNum = 30;

void ADC_init(void) {
	cli();
	//ADMUX = (1 << ADLAR);									// Write one to ADLAR to left adjust the result.
	//ADMUX |= (1 << REFS0);								// AVCC with external capacitor at AREF pin
	ADMUX |= ((1 << REFS0) | (1 << REFS1));					// Internal 1.1V Voltage Reference with external capacitor at AREF pin
	ADCSRA |= ((1 << ADEN) | (1 << ADPS2) | (1 << ADPS1));	// ADC Prescaler Selections --> CLK/64
	ADCSRB = 0x00;											// Free Running mode
	DIDR0 |= (1 << ADC0D) | (1 << ADC1D) | (1 << ADC2D); 	// The digital input buffer on the corresponding ADC pin is disabled
	sei();
}

//function receives as argument a channel number and returns ADC input value
uint16_t ADC_result(unsigned char adcChnl) {
	//cli(); // this brokes i2c!
	ADMUX = adcChnl | (ADMUX & 0xF0);						//select channel
	_delay_us(10);											//delay to stabilize input voltage
	ADCSRA |= (1 << ADSC);									// start conversion (ADSC = 1)
	while((ADCSRA & 0x10)==0);								// wait during conversion (ADIF = 1 when conversion complete)
	ADCSRA |= (1 << ADIF);									//clear ADIF: ADC Interrupt Flag
	return ADCW;											// ADCW - contains ADCH and ADCL
	//sei();
}

//function calculates average ADC value for N samples
uint16_t ADC_result_average(unsigned char adcChnl) {
	static uint16_t adcSum;
	adcSum = 0;
	
	for (unsigned char i=0; i < samplesNum; i++)
	{
		adcSum += ADC_result(adcChnl);		
	}
	return (adcSum / samplesNum);
}

void setAverage(uint8_t inptVal) {	//Set number of samples
	samplesNum = inptVal;
}