/**
* \file
*
* \brief Empty user application template
*
*/

/**
* \mainpage User Application template doxygen documentation
*
* \par Empty user application template
*
* Bare minimum empty user application template
*
* \par Content
*
* -# Include the ASF header files (through asf.h)
* -# "Insert system clock initialization code here" comment
* -# Minimal main function that starts with a call to board_init()
* -# "Insert application code here" comment
*
*/

/*
* Include header files for all drivers that have been imported from
* Atmel Software Framework (ASF).
*/

/*****************************************************************************
*
* Atmel Corporation
*
* File : main.c
* Compiler : IAR EWAAVR 2.28a/3.10c
* Revision : $Revision: 2516 $
* Date : $Date: 2007-09-27 10:41:15 +0200 (to, 27 sep 2007) $
* Updated by : $Author: mlarsson $
*
* Support mail : avr@atmel.com
*
* Supported devices : All devices with a TWI module can be used.
* The example is written for the ATmega16
*
* AppNote : AVR311 - TWI Slave Implementation
*
* Description : Example of how to use the driver for TWI slave
* communication.
*
****************************************************************************/
/*! \page MISRA
*
* General disabling of MISRA rules:
* * (MISRA C rule 1) compiler is configured to allow extensions
* * (MISRA C rule 111) bit fields shall only be defined to be of type unsigned int or signed int
* * (MISRA C rule 37) bitwise operations shall not be performed on signed integer types
* As it does not work well with 8bit architecture and/or IAR
* Other disabled MISRA rules
* * (MISRA C rule 109) use of union - overlapping storage shall not be used
* * (MISRA C rule 61) every non-empty case clause in a switch statement shall be terminated with a break statement
*/

#include <stdio.h>
#include <string.h>
#include "port_init.h"
#include "timer.h"
#include "ADC.h"
#include "TWI_slave.h"
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>	

int main( void )
{
	port_init();
	timerConfig();
	ADC_init();
	
	// Initialise TWI module for slave operation. Include address and/or enable General Call.
	TWI_Slave_Initialise(TWI_SLV_ADDR);
	sei();
	// Start the TWI transceiver to enable reception of the first command from the TWI Master.	
	TWI_Start_Transceiver();
	
	// This example is made to work together with the AVR315 TWI Master application note. In addition to connecting the TWI
	// pins, also connect PORTB to the LEDS. The code reads a message as a TWI slave and acts according to if it is a
	// general call, or an address call. If it is an address call, then the first byte is considered a command byte and
	// it then responds differently according to the commands.
	// This loop runs forever. If the TWI is busy the execution will just continue doing other operations.
	
	while (1)
	{}
}

