/*
* port_init.c
*
* Created: 09.01.2015 09:26:39
*  Author: Stanislav Chystiakov (DESY)
*/
#include <stdio.h>
#include <string.h>
#include <avr/io.h>
#include "port_init.h"
#include "timer.h"


void port_init() {		// Port C, D initialization
	DDRB |= 0x03;		// PORTB PIN_PB0 and PIN_PB1 out
	PORTB &= 0xFC;		// PIN_PB0 and PIN_PB1 = 0
	//PORTB |= 0x03;		// PIN_PB0 and PIN_PB1 = 1
	
	DDRC |= 0x08;		// PORTC out (MCULED)
	PORTC |= 0x08;		// MCULED off
	
	DDRD = 0xFF;		// PORTD out
	PORTD = 0x00;
}

void setPortDirection(uint8_t portName, uint8_t portDir) { // Set port direction
	switch (portName) {	
		case _PORTB_:
		{
			DDRB = (PORT_B_SAFE_MASK & portDir); // allow to change only PIN_PB0 and PIN_PB1
			break;
		}
		case _PORTD_:
		{
			DDRD = portDir;
			break;
		}
		case _PORTC_: // Test only!
		{
			break; // it is not possible to change preset port C direction
		}
	}
}

void setPortLevel(uint8_t portName, uint8_t portLev) { // Set port level
	switch (portName) {
		case _PORTB_:
		{
			PORTB = (PORT_B_SAFE_MASK & portLev); // allow to change only PIN_PB0 and PIN_PB1
			break;
		}
		case _PORTD_:
		{
			PORTD = portLev;
			break;
		}
		case _PORTC_: //Test only!
		{
			PORTC = (PORT_C_SAFE_MASK & portLev); // allow to change only MCULED_PIN
			break;
		}
	}
}

uint8_t readPortLevel(uint8_t portName) { // Read port level
	uint8_t portLevel = 0;
	switch (portName) {
		case _PORTB_:
		{
			portLevel = PORTB;
			break;
		}
		case _PORTD_:
		{
			portLevel = PORTD;
			break;
		}
		case _PORTC_:
		{
			portLevel = PORTC;
			break;
		}
	}
	return portLevel;
}

char togglePin(char pinName, char pinNum) {
	static char togglePinStatus;
	switch (pinName) {
		case 'B':
		PORTB  ^= pinNum;
		togglePinStatus = TOGGLE_PIN_OK;
		break;
		case 'C':
		PORTC  ^= pinNum;
		togglePinStatus = TOGGLE_PIN_OK;
		break;
		case 'D':
		PORTD  ^= pinNum;
		togglePinStatus = TOGGLE_PIN_OK;
		break;
		default:
		togglePinStatus = TOGGLE_PIN_ERR;
	}
	return togglePinStatus;
}