﻿/*
 * bootloader.c
 *
 * Created: 2015-04-12 13:07:41
 *  Author: Paweł
 */ 
#include "bootloader.h"
#include "timer_lib.h"
#include "usart_lib.h"
#include <avr/eeprom.h>
#include <asf.h>
#include <ccp.h>

BootloaderState state = INIT;
uint8_t page_nr = 0;
uint8_t page[FLASH_PAGE_SIZE] = {0};
uint8_t checksum = 0;
uint8_t handshake = 0;

void start_firmware()
{
	void (*app_start)( void ) = APP_SECTION_START; // Set up function pointer to RESET vector.
	uint8_t temp;
	
	/*Set interrupt vector location to application section of flash */
	temp = PMIC.CTRL & ~PMIC_IVSEL_bm;
	CCP = CCP_IOREG_gc;
	PMIC.CTRL = temp;
	
	EIND = 0x00; /* disable extended indirect jump */
	
	app_start(); /* jump to application */
}

void start_bootloader()
{
	uint8_t chr = 0, temp, error_counter = 0;
	bool err = false, escape = false;	
	while(true)
	{
		switch (state)
		{
			case INIT:
				if (eeprom_read_upgrade() == false)
				{
					usart_putchar(USART_SERIAL, eeprom_read_byte(EEPROM_ADDR_FWUPGRADE));
					start_firmware();
				}
				state = HANDSHAKE;
			break;
			
			///////////////////////////////////////
			case HANDSHAKE:

				do 
				{
					usart_serial_getchar(USART_SERIAL, &chr);
				} while (chr != BOOTLDR_HANDSHAKE);
				usart_serial_getchar(USART_SERIAL, &handshake);
				usart_putchar(USART_SERIAL, BOOTLDR_RES_OK);
				usart_putchar(USART_SERIAL, handshake);
				timer_reset_counter();
				state = DATA;
			break;
			
			//////////////////////////////////////
			case DATA:
				checksum = 0;
				err = false;
				do
				{
					usart_serial_getchar(USART_SERIAL, &chr);
				} while (chr != handshake);
				//checksum_add_byte(chr);
				//command - data, finish, reset
				usart_serial_getchar(USART_SERIAL, &chr);
				checksum_add_byte(chr);		
				if (chr == BOOTLDR_DATA)
				{
					usart_serial_getchar(USART_SERIAL, &page_nr);
					checksum_add_byte(page_nr);
					for (int i = 0; i < FLASH_PAGE_SIZE; i++)
					{
						usart_serial_getchar(USART_SERIAL, &chr);
						checksum_add_byte(chr);						
						if (!escape && chr == BOOTLDR_ESCAPE)
						{
							i--;
							escape = true;
							continue;
						}
						//check for forbidden char
						else if (!escape && (chr == BOOTLDR_DATA || chr == BOOTLDR_HANDSHAKE || chr == BOOTLDR_FINISH || chr == BOOTLDR_RESET))
						{
							usart_putchar(USART_SERIAL, 0x55);
							err = true;
							error_counter++;
						}
						else
							escape = false;
						if (!escape)
							page[i] = chr;
						escape = false;

						timer_reset_counter();
					}
					if (read_compare_checksum())
					{	
						//usart_putchar(USART_SERIAL, BOOTLDR_RES_OK);
						//for (int i = 0; i < 512; ++i)
						//	usart_putchar(USART_SERIAL, page[i]);
						//TODO write page to memory
					 nvm_flash_erase_and_write_buffer(page_nr * FLASH_PAGE_SIZE, page, FLASH_PAGE_SIZE, true);

					}
					else
					{
						usart_putchar(USART_SERIAL, 0x66);
						usart_putchar(USART_SERIAL, checksum);
						err = true;
						error_counter++;
					}
				}
				else if (chr == BOOTLDR_FINISH)
				{
					if (read_compare_checksum())
					{
						state = FINISH;
					}
					else
					{
						err = true;
						error_counter++;	
					}
				}
				else if (chr == BOOTLDR_RESET)
				{
					if (read_compare_checksum())
						ccp_write_io((void *)&RST.CTRL, RST_SWRST_bm);
					else
					{
						err = true;
						error_counter++;
					}		
				}
				else
				{
					err = true;
					error_counter++;					
				}
				
				if (!err)
				{
					usart_putchar(USART_SERIAL, BOOTLDR_RES_OK);
					timer_reset_counter();
				}
				else
				{
					if (error_counter < BOOTLDR_ERR_MAX)
						usart_putchar(USART_SERIAL, BOOTLDR_RES_FAIL);
					else
					{
						usart_putchar(USART_SERIAL, BOOTLDR_RES_ABORT);
						printString("Resetting CPU - too many errors");
						ccp_write_io((void *)&RST.CTRL, RST_SWRST_bm);
					}
				}
		
		
			break;
			
			////////////////////////////////////
			case FINISH:
				//eeprom_write_ready();
				start_firmware();
			break;
			default:
			break;
		}
	}
}

void checksum_add_byte(uint8_t byte)
{
	checksum = (checksum ^ byte) % 256;
}

void eeprom_write_upgrade( void )
{
	eeprom_write_byte(EEPROM_ADDR_FWUPGRADE, EEPROM_FWUPGRADE_VALUE);	
}

void eeprom_write_ready( void )
{
	eeprom_write_byte(EEPROM_ADDR_FWUPGRADE, EEPROM_FWREADY_VALUE);
}

bool eeprom_read_upgrade( void )
{
	//return false;
	// return true;
	//usart_putchar(USART_SERIAL, eeprom_read_byte((EEPROM_ADDR_FWUPGRADE)));
	return eeprom_read_byte(EEPROM_ADDR_FWUPGRADE) != EEPROM_FWREADY_VALUE;
}

bool read_compare_checksum(void)
{
	uint8_t chr;
	usart_serial_getchar(USART_SERIAL, &chr);	
	return (chr == checksum);	
}
