/*
* fan_current_monitor_lib.c
*
* Created: 09.02.2015 14:10:07
*  Author: schystia
*/

#include "i2c_lib.h"
#include "twim.h"
#include "mgmt_protocol_master.h"
#include "usart_lib.h"
#include "telnet_protocol.h"
#include "power_monitor.h"

#include "fan_current_monitor_lib.h"
#include "cpld_protocol.h"
#include "fan_lib.h"
#include <asf.h>
#include <util/delay.h>
#include <stdio.h>

uint8_t portName = 0; // selected port to modify



int FREDFAN_Board_CheckPresence(void)
{
	uint8_t reg;
	uint8_t read_data[2] = {0,0};
	char message[80];
	reg = CMD_TWI_CHECK_PRESENCE;
	if ( i2c_read(&MANAGE_I2C, FREDFAN_ADDR,  &reg, 1, read_data, 2) == TWI_SUCCESS) {
		sprintf(message, "FRED Fan Ver. %d Rev. %d\r\n", read_data[0], read_data[1]);
		printString(message);
		return STATUS_OK;
	}
	else {
		printString("FRED Fan board not detected!\r\n");
		return ERR_IO_ERROR;
	}
}

void readFanCurrentAverageAll(void){	// read all 3 ADC Channels
										// ATmega88PA samples N ADC values for each channel and calculates average value
	uint8_t addr, reg;
	int data;
	uint8_t shift, i;
	uint8_t read_data[4] = {0,0,0,0};
	float current;
	char message[80];
	addr = FREDFAN_ADDR;
	reg = CMD_TWI_READ_ADC_ALL;
	
	if(i2c_read(&MANAGE_I2C, addr, &reg, 1, read_data, 4) == TWI_SUCCESS)
	{
		shift = 2;
		for(i = 0; i < 3 ; i++) {
			data = read_data[i] + ((read_data[3] << shift) & 0x0300);
			shift += 2;
			current = data * CURRENT_SCALE;
			sprintf(message, "Average current value of FAN[%d] = %d mA\r\n", i, (int)current);
			printString(message);
		}
	}
	else
	{
		printString("Failure reading values!\r\n");
	}
}

int readFanCurrent( uint8_t fan, unsigned *current )
{
	uint8_t addr, reg;
	int data;
	uint8_t read_data[4] = {0,0,0,0};
	addr = FREDFAN_ADDR;
	reg = CMD_TWI_READ_ADC_ALL;
	
	if(i2c_read(&MANAGE_I2C, addr, &reg, 1, read_data, 4) == TWI_SUCCESS)
	{
		data = read_data[fan] + ((read_data[3] << (2*(fan+1))) & 0x0300);
		*current = (int) data * CURRENT_SCALE;
		return STATUS_OK;
	}
	else
	{
		return ERR_IO_ERROR;
	}
}


int resetTimeToErr(void) // watch dog
{
	static uint8_t printEnable = 1;	
	char message[80];
	uint8_t status, addr, reg, data;
	addr = FREDFAN_ADDR;
	reg = CMD_RESET_ERR_TIME;
	data = 0;
	
	if(i2c_write(&MANAGE_I2C, addr,  &reg, 1, &data, 1) == TWI_SUCCESS)
	{
		readFanAlarmStatusReg(&status);
		if(status != 0) {
			sprintf(message, "Alarm Status Register = 0x%X\r\n", status);
			printString(message);
			uint8_t alarm;
			readFanAlarmStatusReg(&alarm);
			if(alarm == 0) {
				setOperatingMode(FANCONFIG_OPENLOOP);
				printString("FAN error has been fixed!\r\n");
			}
		}
		printEnable = 1;
		return STATUS_OK;
	}
	else {
		if (printEnable == 1)
		printString("Access to ATmega88PA error!\r\nWatchdog reset error!\r\n");
		printEnable = 0;
		return ERR_IO_ERROR;		
	}
}

int setAverage(uint8_t inptVal) // set average for ADC (number of samples)
{
	uint8_t addr, reg, data;
	addr = FREDFAN_ADDR;
	reg = CMD_TWI_SET_AVERAGE;
	data = inptVal;
	
	if(i2c_write(&MANAGE_I2C, addr,  &reg, 1, &data, 1) == TWI_SUCCESS)
	{
		return STATUS_OK;
	}
	else {
		printString("FRED Fan board not detected!\r\n");
		return ERR_IO_ERROR;
	}
}

void selectPort(uint8_t inptVal)
{
	portName = inptVal;
}

int setPortDirection(uint8_t port, uint8_t portDir)
{
	uint8_t addr, reg;
	addr = FREDFAN_ADDR;
	reg = CMD_TWI_SET_PORT_DIR;
	uint8_t transm_data[2] = {0};
	
	transm_data[0] = portName;
	transm_data[1] = portDir;
	
	if (portName != 0) {
		if(i2c_write(&MANAGE_I2C, addr,  &reg, 1, transm_data, 2) == TWI_SUCCESS)
		{
			printString("Success\r\n");
			return STATUS_OK;
		}
		else {
			printString("FRED Fan board not detected!\r\n");
			return ERR_IO_ERROR;
		}
	}
	else
	{
		printString("You should first select the port!\r\n");
		return ERR_IO_ERROR;
	}
}

int setPortLevel(uint8_t port, uint8_t portLev)
{
	uint8_t addr, reg;
	addr = FREDFAN_ADDR;
	reg = CMD_TWI_SET_PORT_LEV;
	uint8_t transm_data[2] = {0,0};
	
	transm_data[0] = port;
	transm_data[1] = portLev;

	if(i2c_write(&MANAGE_I2C, addr,  &reg, 1, transm_data, 2) == TWI_SUCCESS)
	{
		return STATUS_OK;
	}
	else {
		printString("FRED Fan board not detected!\r\n");
		return ERR_IO_ERROR;
	}
}

int readPortLevel(uint8_t port)
{
	uint8_t addr, reg, data;
	char message[80];
	addr = FREDFAN_ADDR;
	reg = CMD_TWI_READ_PORT_LEV;
	data = port;
	
	if(i2c_write(&MANAGE_I2C, addr,  &reg, 1, &data, 1) == TWI_SUCCESS)
	{
		if(i2c_read(&MANAGE_I2C, addr, &reg, 1, &data, 1) == TWI_SUCCESS)
		{
			sprintf(message, "0x%X\r\n", data);
			printString(message);
			return STATUS_OK;
		}
		else {
			printString("FRED Fan board not detected!\r\n");
			return ERR_IO_ERROR;
		}
	}
	return -1;
}
