/*
 * fan_current_monitor_lib.h
 *
 * Created: 09.02.2015 14:10:17
 *  Author: schystia
 */ 

#ifndef FAN_CURRENT_MONITOR_LIB_H_
#define FAN_CURRENT_MONITOR_LIB_H_
#include <stdbool.h>

#define ADC_SCALE					1.07421875F		// (1100mV / 2^10)	[ 1,1V / 2^10 = 0,00107421875 ]
#define CURRENT_SCALE				2.1484375F		// (1100mV / 2^10) / (10 * 0,05Ohm)


#define FREDFAN_ADDR				0x0C >> 1 // FRED Fan board address

#define ADC_CH0						0x00	// FAN0
#define ADC_CH1						0x01	// FAN1
#define ADC_CH2						0x02	// FAN2
//#define CMD_TWI_WRITE				0x10
#define CMD_TWI_READ_ADC			0x20	// read only 1 ADC channel
//#define CMD_TWI_READ_ADC_AVRG		0x21	// read average only for 1 ADC channel (calculated by ATmega88PA)
#define CMD_TWI_READ_ADC_ALL		0x22	// read average for all channels (calculated by ATmega88PA)
#define CMD_TWI_READ_VAL			0x23	// read value from ATmega88PA (Test ONLY!!!)
#define CMD_TWI_CHECK_PRESENCE		0x24	// read FRED Fan board version and revision
#define CMD_TWI_SET_AVERAGE			0x25	// set number of ADC samples
#define CMD_TWI_SET_PORT_DIR		0x26	// set port direction /setPortDirection()
#define CMD_TWI_SET_PORT_LEV		0x27	// set port level /setPortLevel()
#define CMD_TWI_READ_PORT_LEV		0x28	// read Port B level /readPortLevel()

#define CMD_RESET_ERR_TIME			0x30	// reset error time

#define _PORTB_						0x41	// transmit value for PORTB
#define _PORTD_						0x42	// transmit value for PORTD
#define _PORTC_						0x43	// transmit value for PORTC


void fredfan_init(void);
int FREDFAN_Board_CheckPresence(void);
int resetTimeToErr(void);

int readFanCurrent(uint8_t fan, unsigned *current);
void readFanCurrentAverage(uint8_t adcChnl);
void readFanCurrentAverageAll(void);
int setAverage(uint8_t inptVal);
void selectPort(uint8_t inptVal);
int setPortDirection(uint8_t port, uint8_t portDir);
int setPortLevel(uint8_t port, uint8_t portLev);
int readPortLevel(uint8_t port);

#endif /* FAN_CURRENT_MONITOR_LIB_H_ */




