﻿/**
 * @file power_monitor.h
 * @brief Header for library for communication with power monitors and handling channel info
 *
 * @copyright (C) 2014 Lodz University of Technology, Lodz, Poland  (TUL)
 * @copyright (C) 2014 Deutsches Elektronen-Synchrotron, Hamburg, Germany (DESY)
 * @author Pawel Plewinski (TUL)
 * @author Piotr Perek (TUL)
 * @author Dariusz Makowski (TUL)
 * @par License
 *
 * Redistribution and use in source and binary forms, with or without
 * modification is not allowed. Exceptions are regulated by separate License document. In all cases it is requested to provide the above header.
 *
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */


#ifndef POWER_MONITOR_H_
#define POWER_MONITOR_H_



#include <stdint.h>
#include <stdbool.h>
#include "user_board/user_board.h"


#define PM_BUSY_FLAG (1 << 3)

#define OVERCURRENT_LIMIT_COUNTER 20


extern uint8_t reg_control_addr;
extern uint8_t reg_control_data_continuous;
extern uint8_t reg_control_data_snapshot;

/**
\brief Structure storing information about a channel
*/
typedef struct channel_s
{
	uint32_t voltage; /** Current voltage */
	uint32_t current; /** Current current */
	uint32_t power; /** Current power */
	uint8_t i2c_addr; /** Address of a monitor on i2c bus */
	uint32_t *eeprom_addr; /** Address of current limit in EEPROM **/
	bool state; /** Desired state of a channel */
	bool actual_state; /** Actual state of a channel */
	bool state_changed;  /** Indicates whether state of a channel has recently changed */
	bool alarm_mask; /** Alarm mask for a channel*/
	bool overcurrent_alarm_status; /** Overcurrent alarm status for a channel*/
	unsigned overcurrent_alarm_counter; /** Overcurrent alarm status for a channel*/
	bool alarm_pin0_status; /** Alarm status for pin0*/
	bool alarm_pin1_status; /** Alarm status for pin1*/	
	bool negative; /** Indicates whether voltage of a channel is negative*/
	bool i2c_error; /** Indicates error when reading state from i2c*/
	uint32_t current_limit; /**Current limit*/
} RelayChannel;

extern RelayChannel channels[CHANNEL_NUMBER];

void initChannelStructures(void);
void pollChannel(RelayChannel *channel, int nr, bool timerTrigerred);
void pollChannels(bool timerTriggerred);

uint8_t getPowerState(void);
uint8_t getAlarmMask(void);
void setAlarmMask(uint8_t alarmMask);
void setCurrentLimit(int channel, uint32_t limit);
void clearOvercurrentAlarm (int channel);

void readRelays(void);
void writeRelays(void);

#endif /* POWER_MONITOR_H_ */