﻿/**
 * @file register_test.c
 * @brief Test of communication between CPLD and uC
 *
 * @copyright (C) 2014 Lodz University of Technology, Lodz, Poland  (TUL)
 * @copyright (C) 2014 Deutsches Elektronen-Synchrotron, Hamburg, Germany (DESY)
 * @author Pawel Plewinski (TUL)
 * @author Dariusz Makowski (TUL)
 * @par License
 *
 * Redistribution and use in source and binary forms, with or without
 * modification is not allowed. Exceptions are regulated by separate License document. In all cases it is requested to provide the above header.
 *
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "register_test.h"
#include "usart_lib.h"
#include "power_monitor.h"
#include "telnet_protocol.h"
#include "cpld_protocol.h"

#include <asf.h>
#include <util/delay.h>
#include <stdio.h>

/**
 * \brief Test of CPLD registers - read and print states
 * 
 * 
 * \return void
 */
void testRegisters()
{

	printPString("START\r\n");

	for (int i = 0; i < 8; ++i)
	{
		volatile uint8_t reg = readHalfRegister(i, 0);
		char x[5];
		sprintf(x, "%d 0 %X\r\n", i,  reg);
		printString(x);
		
		asm volatile("nop");
		
		reg = readHalfRegister(i, 1);
		sprintf(x, "%d 1 %X\r\n", i, reg);
		printString(x);
	}


	for (int i = 0; i < CHANNEL_NUMBER; ++i)
	{
		usart_serial_putchar(USART_SERIAL,'0'+i);
		writeRegister(CPLD_ADDR_RELAY_CONTROL, 1 <<i);
		char buf[20];
		sprintf(buf, "\r\n%2X %2X\r\n",
		readRegister(CPLD_ADDR_RELAY_CONTROL),
		readRegister(CPLD_ADDR_RELAY_STATUS));
		printString(buf);
		_delay_ms(1000);
	}

	while(1)
	;

}