﻿/**
 * @file usart_lib.C
 * @brief USART support library
 *
 * @copyright (C) 2014 Lodz University of Technology, Lodz, Poland  (TUL)
 * @copyright (C) 2014 Deutsches Elektronen-Synchrotron, Hamburg, Germany (DESY)
 * @author Pawel Plewinski (TUL)
 * @author Piotr Perek (TUL)
 * @author Dariusz Makowski (TUL)
 * @par License
 *
 * Redistribution and use in source and binary forms, with or without
 * modification is not allowed. Exceptions are regulated by separate License document. In all cases it is requested to provide the above header.
 *
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <usart.h>
#include <usart_serial.h>
#include <conf_usart_serial.h>
#include <asf.h>
#include <ctype.h>
#include "usart_lib.h"
#include "config/conf_usart_serial.h"
#include <stdio.h>
#include "telnet_protocol.h"
#include <progmem.h>
#include <util/delay.h>

volatile bool usartEcho = false;

/**
 * \brief Initialize UART
 * 
 * \param 
 * 
 * \return void
 */
void usart_init(void)
{
	//cli();
	//sysclk_init();
	static usart_serial_options_t usart_options = {
		.baudrate = USART_SERIAL_BAUDRATE,
		.charlength = USART_SERIAL_CHAR_LENGTH,
		.paritytype = USART_SERIAL_PARITY,
		.stopbits = USART_SERIAL_STOP_BIT
	};

	usart_serial_init(USART_SERIAL, &usart_options);
	usart_set_rx_interrupt_level(USART_SERIAL, PMIC_LOLVLEN_bm);
	printString("HELLO_v1");
}

/**
 * \brief UART interrupt
 */
ISR (USARTF0_RXC_vect)
{
	ioport_toggle_pin(MMC_LED);
	uint8_t ch;
	usart_serial_getchar((USART_SERIAL), &ch);
	if (usartEcho)
	{	
		usart_serial_putchar((USART_SERIAL), ch);
		if (ch == '\r')	usart_serial_putchar((USART_SERIAL), '\n');
	}
	telnetPutChar(ch);
}

/**
 * \brief Print 0-terminated string
 * 
 * \param string
 * 
 * \return void
 */
void printString(const char *string )
{
	while (*string)
	{
		usart_serial_putchar(USART_SERIAL, *(string++));
//		_delay_us(1000);
	}
}

void printProgmemString(const char * string)
{
	char ch;
	while ((ch = PROGMEM_READ_BYTE(string++)) != 0)
		usart_serial_putchar(USART_SERIAL, ch);


}
